//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 1/28/2012.
//


#include "TKLock.h"
#include "TKContainer.h"
#include "TKUICell.h"
#include "DNTimeKeeper.h"
#include "DNHTTPServer.h"
#include "DNEngine.h"

DNEngine::DNEngine(TKContainer *container) :
mContainer(container),mIsRunning(false),mPortNumber(9080),mHTTPServer(NULL)
{
    mTimeKeeper = new DNTimeKeeper();

}


bool DNEngine::startHTTPServer(int portNumber)
{
    if (mHTTPServer && mHTTPServer->isRunning())
    {
        mHTTPServer->stop();
        delete mHTTPServer;
        mHTTPServer = NULL;
    }
    mHTTPServer = new DNHTTPServer(this);
    if (mHTTPServer)
    {
        mHTTPServer->setPortNumber(portNumber);
        mHTTPServer->start();
    }
    return true;
}

void DNEngine::stopServer()
{
    if (mHTTPServer && mHTTPServer->isRunning())
    {
        mHTTPServer->stop();
        delete mHTTPServer;
        mHTTPServer = NULL;
    }    
}

void DNEngine::setTickIntervalSec(float interval)
{
    if (mTimeKeeper)
        mTimeKeeper->setIntevalSec(interval);
}

void DNEngine::doTickThread()
{
    mTickThreadLock.lock();
    if (mIsRunning)
    {
        //doTickThread is already running.
        mTickThreadLock.unlock();
        return;
    }
    mIsRunning = true;
    mTickThreadLock.unlock();
    
    
    DNTimeKeeper *timeKeeper = new DNTimeKeeper();
    
    while(mIsRunning)
    {
        mContainer->doTick(timeKeeper->getTickTime());
        timeKeeper->sleepUntilNextInterval();
    }
    
    delete timeKeeper;
}

float DNEngine::doClientGetRequest(const char* path)
{
    TKUICell *cell = (TKUICell*)mContainer->getInterfaceCell(path);
    float result = 0;
    if (cell)
    {
        result = cell->getValue();
    }
    return result;
}

bool DNEngine::doClientSetRequest(const char* path, const char* value)
{
    TKUICell *cell = (TKUICell*)mContainer->getInterfaceCell(path);
    bool result = false;
    if (cell)
    {
        cell->setValue(atof(value));
        result = true;
    }
    return result;
}

std::string DNEngine::getContentPath()
{
    if (mContainer)
        return mContainer->getContentPath();
    else
        return "";
}

