//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 1/7/2012.
//

#include "DNUtils.h"

void trimString(std::string& str)
{
    std::string::size_type pos1 = str.find_first_not_of(' ');
    std::string::size_type pos2 = str.find_last_not_of(' ');
    std::string::size_type pos1_2 = str.find_first_not_of('\t');
    std::string::size_type pos2_2 = str.find_last_not_of('\t');
    
    pos1 = pos1 < pos1_2 ? pos1_2 : pos1;
    pos2 = pos2 < pos2_2 ? pos2_2 : pos2;
    
    str = str.substr(pos1 == std::string::npos ? 0 : pos1, 
                     pos2 == std::string::npos ? str.length() - 1 : pos2 - pos1 + 1);
}

std::string getFQNString(const char *location, const char *name)
{
    std::string path;
    std::string node;
    int l = (int)strlen(location) - 1;
    bool f = false;
    while (l >= 0)
    {
        if (location[l] == '/')
        {
            f = true;
            break;
        }
        l--;
    }
    if( f )
    {
        if (l == 0)
        {
            path = "/";
            node = location;
        }
        else
        {
            node = location;
            path = node.substr(0,l);
            node = node.substr(l);
        }
    }
    else
    {
        path = "";
        node = location;
    }
    
    const char *c = name;
    int i = 0;
    const char *p = c;
    while(*c)
    {
        if (*c == '/')
        {
            if (i >= 2 && *(c - 2) == '.' && *(c - 1) == '.')
            {
                unsigned long l = path.find_last_of("/");
                if( l != std::string::npos )
                {
                    if (l == 0)
                    {
                        path = "/";
                    }
                    else if (l > 0)
                    {
                        path = path.substr(0, l);
                    }
                }
                else
                {
                    path = "";
                }
                p = c;
            }
            else if (i >= 1 && *(c - 1) == '.')
            {
                p = c;
            }
            else if (i >= 1 && *(c - 1) == '/')
            {
                p = c; 
            }
            else if (i == 0)
            {
                if (path.length() > 0 && path.at(0) == '/')
                {
                    path = "/";
                }
                else
                {
                    path = "";
                }
            }
            else
            {
                size_t pl = path.length();
                if (pl == 0)
                {
                    if (*p == '/')
                    {
                        p++;
                    }                    
                }
                else if (pl == 1)
                {
                    char p1 = path.at(0);
                    if (p1 == '/' || p1 == '.')
                    {
                        if (*p == '/')
                        {
                            p++;
                        }
                    }
                }
                else
                {
                    if (*p != '/')
                    {
                        path.append("/");
                    }
                }
                std::string s(p, c-p); 
                path.append(s);
                p = c;
            }
        }
        c++;
        i++;
        if (i > 1024) break;
    }
    
    if (*p == '/')
    {
        node = p;
    }
    else if(*p == '#')
    {
        node.append(p);
    }
    else
    {
        const char *p2 = p;
        bool hasAnchor = false;
        while(*p2)
        {
            if (*p2 == '#')
            {
                hasAnchor = true;
                break;
            }
            p2++;
        }
        if (hasAnchor)
        {
            node = "/";
            node.append(p);        
        }
        else
        {
            node.append("#");
            node.append(p);
        }
    }
    
    int pl = (int)path.length();
    std::string fqn;
    if (pl == 0)
    {
        if (node.at(0) == '/' && node.length() > 1)
        {
            fqn = node.substr(1);
        }
        else
        {
            fqn = node;
        }
        
    }
    else if (pl == 1)
    {
        if (path.at(0) == '/' && node.length() > 0 && node.at(0) == '/')
        {
            fqn = node;
        }
        else
        {
            fqn = path.append(node);
        }
    }
    else
    {
        fqn = path.append(node);
    }
    
    return fqn;
}

std::string getJSEscapeString(const char *cstring)
{
    const char *c = cstring; 
    int len = 0;
    while(*c)
    {
        if (*c == '/') len++;
        else if (*c == '_') len++;
        else if (*c == '.') len++;
        else if (*c == '#') len++;
                 
        c++;
        len++;
    }
    char *buf = (char*)malloc(len);
    char *result = buf;
    
    c = cstring; 
    while(*c)
    {
        if (*c == '/')
        {
            *buf = '_';
            buf++;
            *buf = 's';
            buf++;
        }
        else if (*c == '_')
        {
            *buf = '_';
            buf++;
            *buf = '_';
            buf++;            
        }
        else if (*c == '.')
        {
            *buf = '_';
            buf++;
            *buf = 'd';
            buf++;
        }
        else if (*c == '#')
        {
            *buf = '_';
            buf++;
            *buf = 'S';
            buf++;
        }
        else
        {
            *buf = *c;
            buf++;
        }
        
        c++;
        len++;
    }
    *buf = 0;
    
    return std::string(result);
}

