//  Copyright (c) 2012 Dennco Project
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

//
//  Created by tkawata on 12/12/2011.
//

#include "TKJSCellCode.h"
#include "TKJSContainer.h"
#include "TKLog.h"
#include "TKCell.h"
#include "TKJSCellBase.h"
#include "TKJSCellCodeInstance.h"
#include "TKDebug.h"
#include "DNUtils.h"
#include <string>

TKCellCodeInstance* TKJSCellCode::createCellCodeInstance(TKCell *owner, const void *data)
{
    TKCellCodeInstance *result = NULL;
    if (mJsConstructorRef)
    {
        JSContextRef jsCtx = mJSContainer->getJSContext();
        
        JSValueRef api[] = {((TKJSCellBase*)owner)->getCellAPI()};
        
        JSObjectRef jobj = JSObjectCallAsConstructor(mJSContainer->getJSContext(), mJsConstructorRef, 1, api, 0);
        result = new TKJSCellCodeInstance(owner, jobj);

        // execute init code 1
        const char * initCode = (char*)data;
        JSStringRef jstrScript = JSStringCreateWithUTF8CString(initCode);
        JSEvaluateScript(jsCtx, jstrScript, jobj, NULL, 1, NULL);
        JSStringRelease(jstrScript);
        
    }
    
    return result;
}

TKJSCellCode::~TKJSCellCode()
{    
}

TKJSCellCode::TKJSCellCode(std::string name, std::string cellapi, TKJSContainer *container, std::string code):
    TKCellCode(name, cellapi),
    mJsConstructorRef(NULL),
    mJSContainer(container)
{
   //parse input js string here..
    
#ifdef DEBUG
    DEBUG_TRACE("TKJSCellCode::TKJSCellCode() parse for %s\n=== Original code =====\n%s\n", name.c_str(), code.c_str());
#endif
    const char *c = code.c_str();
        
    int  inComment1 = 0;   //  //
    int  inComment2 = 0;   //  /* ... */
    int  inComment2_end = 0;
    int  inFunction = 0;
    int  inVar = 0;
    int  inQuote = 0;
    int  inDoubleQuote = 0;
    int  nest = 0;
    const char *sp = 0;
    int  l = 0;
    int  lp = 0;
    std::string funcs = "";
    std::string vars = "";
    std::string fff = "";
    std::string jname = getJSEscapeString(name.c_str());
    
    const char* str_comment1 = "//";
    const char* str_comment2 = "/*";
    const char* str_comment2_end = "*/";
    const char* str_function = "function";
    const char* str_var = "var";
    
    while (*c)
    {
        
        if (*c == '\r' || *c == '\n')
        {
            lp = 0;
        }
        else if (*c != ' ' && *c != '\t')
        {
            lp++;
        }
        
        if (inComment1 == -1)
        {
            if (*c == '\r' || *c == '\n' )
                inComment1 = 0;
            c++;
            continue;
        }
        else if (inComment2 == -1)
        {
            if (*c == str_comment2_end[inComment2_end]) 
            {
                inComment2_end ++;
                if (str_comment2_end[inComment2_end] == 0) 
                {
                    inComment2 = 0;
                    inComment2_end = 0;
                }
            }
            else
            {
                inComment2_end = 0;
            }
            c++;
            continue;
        }
        
        if (inQuote == -1)
        {
            if (*c == '\\')
            {
                c++;
                continue;
            }
            if (*c == '\'')
            {
                inQuote = 0;
            }
            c++;
            continue;
        }
        else if (inDoubleQuote == -1)
        {
            if (*c == '\\')
            {
                c++;
                continue;
            }
            if (*c == '\"')
            {
                inDoubleQuote = 0;
            }
            c++;
            continue;
        }
        
        if (*c == '\'')
        {
            inQuote = -1;
            c++;
            continue;
        }

        if (*c == '\"')
        {
            inDoubleQuote = -1;
            c++;
            continue;
        }

        //IN COMMENT?
        if (*c == str_comment1[inComment1])
        {
            inComment1++;
            if (str_comment1[inComment1] == 0)
            {
                inComment1 = -1;
                c++;
                continue;
            }
        }
        else
        {
            inComment1 = 0;
        }
      
        if (*c == str_comment2[inComment2])
        {
            inComment2++;
            if (str_comment2[inComment2] == 0)
            {
                inComment2 = -1;
                c++;
                continue;
            }
        }
        else
        {
            inComment2 = 0;
        }
        
        if (!inComment1 && !inComment2 && !inQuote && !inDoubleQuote && *c == '{') 
        {
            nest++;
            if (inFunction == -1 && nest == 1)
            {
                fff = "";
                fff.append(sp,c-sp+1);
                unsigned long fp = fff.find( "(", 0 );
                if( fp != std::string::npos )
                {
                    fff = fff.substr(0, fp);
                    trimString(fff);
                    sp += fp;
                }
                else
                {
                    inFunction  = 0;
                }
            }
        }
        else if (!inComment1 && !inComment2 && !inQuote && !inDoubleQuote && *c == '}')
        {
            nest--;
            if (nest == 0)
            {
                if (inFunction == -1)
                {
                    //FUNCTION DEF
                    std::string fn = "";
                    funcs.append(jname);
                    funcs.append(".prototype.");
                    funcs.append(fff);
                    funcs.append(" = function");
                    funcs.append(sp ,c-sp+1);
                    funcs.append("\r\n");
                    inFunction = 0;
                }
            }
            c++;
            continue;
        }
        
        if (*c == ';')
        {
            if (inVar == -1)
            {
                //VAR DEF
                std::string t = "";
                t.append(sp,c-sp+1);
                trimString(t);
                vars.append("this.");
                vars.append(t);
                vars.append("\r\n");
                inVar = 0;
            }
            c++;
            continue;
        }
    
        if (inFunction == -1)
        {
            if (l == 0) {
                if (*c != ' ' && *c != '\t' && *c != '\r' && *c != '\n')
                {
                    inFunction =0;
                    c++;
                    l = 0;
                    continue;
                }
            }
            c++;
            l++;
            continue;
        }
                                
        if (inVar == -1)
        {
            if (l == 0) {
                if (*c != ' ' && *c != '\t' && *c != '\r' && *c != '\n' && *c != '=')
                {
                    inVar =0;
                    c++;
                    l = 0;
                    continue;
                }
            }
            c++;
            l++;
            continue;            
        }
        
        
        if (nest == 0 && *c == str_function[inFunction] && (inFunction+1) == lp)
        {
            inFunction++;
            if (str_function[inFunction] == 0)
            {
                inFunction = -1;
                sp = c+1;
                l = 0;
            }
            c++;
            continue;
        }
        
        if (nest == 0 && *c == str_var[inVar] && (inVar+1) == lp)
        {
            inVar++;
            if (str_var[inVar] == 0)
            {
                inVar = -1;
                sp = c+1;
                l = 0;
            }
            c++;
            continue;
        }
        
        c++;

    }
    
    std::string stmt = "";
    stmt.append("function ");
    stmt.append(jname);
    stmt.append("(ownerCell)\n{\nthis.cell = ownerCell;\n");
    stmt.append(vars);
    stmt.append("\n}\n");
    stmt.append(funcs);
#ifdef DEBUG    
    DEBUG_TRACE("\n\n==== Translated JS statement:=====\n%s\n=====\n", stmt.c_str());
#endif

    JSContextRef jsCtx = mJSContainer->getJSContext();
    JSObjectRef  jsGlobalObj = mJSContainer->getJSGlobalObject();
    JSStringRef jstrScript = JSStringCreateWithUTF8CString(stmt.c_str());
    JSEvaluateScript(jsCtx, jstrScript, NULL, NULL, 1, NULL);
    JSStringRef jstrClassName = JSStringCreateWithUTF8CString(jname.c_str());
    mJsConstructorRef = (JSObjectRef) JSObjectGetProperty(jsCtx, jsGlobalObj, jstrClassName,NULL);
    JSStringRelease(jstrClassName);
    if (!mJsConstructorRef)
    {
        TKLog::printf("TKJSCellCode ERROR creating:%s", name.c_str());
    }
    
}

